#!/bin/sh

# patch_install.sh
#
# Copyright 2014 Simone Giustetti - Recco IT
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# -----------------------------------------------------------------------------
# Slackware SlackBuild script
# ===========================
# By:        Simone Giustetti <studiosg@giustetti.net>
# For:       patch_install script for Slackware Linux
# Changelog:
# 1.0       2014/07/13 by Simone Giustetti <studiosg@giustetti.net>
#           * Initial build.
# 1.1       2014/09/15 by Simone Giustetti <studiosg@giustetti.net>
#           * Updated to match changes in other build scripts.
#           * Added code to ignore the third digit set in the Slackware release
#
# Run 'sh patch_install.sh' to download and install all patches listed in the
# Slackware Changelog.
#
# -----------------------------------------------------------------------------

# Log variable declaration & initialization
DATE_TIME=`date +%Y%m%d`
FILE_LOG="/tmp/patch_install-${DATE_TIME}.log"
# Log variable declaration & initialization ends here
# Local path declaration
PKG_TREE_BASE="/home/ftp/pub/Linux/Slackware"
# Patch download script location
SCRIPT_RSYNC_SOURCE="http://www.slackware.com/~alien/tools/rsync_slackware_patches.sh"
SCRIPT_RSYNC_DESTINATION="/usr/local/bin/rsync_slackware_patches.sh"
# GPG key download location
GPG_KEY_SOURCE="http://slackware.com/gpg-key"

# Determine target architecture
if [ -z "${MARCH}" ]; then
   MARCH=`uname -m`
   case "${MARCH}" in
      i?86)
         export ARCH=i486
      ;;
      arm*)
         export ARCH=arm
      ;;
      *)
         export ARCH=${MARCH}
      ;;
   esac
fi

echo -e "\E[0;32m+-----------------------+\E[0;0m"
echo -e "\E[0;32m|     ARCH=${ARCH}      |\E[0;0m"
echo -e "\E[0;32m+-----------------------+\E[0;0m"

case "${ARCH}" in
   arm)
      SLACKWARE_VERSION="slackware"
   ;;
   i486)
      SLACKWARE_VERSION="slackware"
   ;;
   i686)
      SLACKWARE_VERSION="slackware"
   ;;
   powerpc)
      SLACKWARE_VERSION="slackware"
   ;;
   s390)
      SLACKWARE_VERSION="slackware"
   ;;
   x86_64)
      SLACKWARE_VERSION="slackware64"
   ;;
esac

# Determine Slackware release number
SLACKWARE_RELEASE=`cat /etc/slackware-version | awk '{ print $NF }' | cut -d\. -f1,2`

# Lets start logging something
echo "${DATE_TIME} - Patch installation started" | tee ${FILE_LOG}
echo "Patch install script for Slackware ${SLACKWARE_RELEASE} (${ARCH})" \
    | tee -a ${FILE_LOG}

# Build patches destination path and check for its presence
PKG_TREE_ROOT="${PKG_TREE_BASE}/${SLACKWARE_VERSION}-${SLACKWARE_RELEASE}/patches"
if [[ ! -d ${PKG_TREE_ROOT} ]]; then
   # Create the directory where to download patches
   echo "Creating directory ${PKG_TREE_ROOT}" | tee -a ${FILE_LOG}
   mkdir -p ${PKG_TREE_ROOT} | tee -a ${FILE_LOG} 2>&1
fi

# Download the standard upgrade script and change its permissions
echo "Downloading the standard patch install script:
   ${SCRIPT_RSYNC_SOURCE}
   to
   ${SCRIPT_RSYNC_DESTINATION}" | tee -a ${FILE_LOG}
wget ${SCRIPT_RSYNC_SOURCE} -O ${SCRIPT_RSYNC_DESTINATION} | tee -a ${FILE_LOG} 2>&1
chmod 0500 ${SCRIPT_RSYNC_DESTINATION} | tee -a ${FILE_LOG} 2>&1

# Run the script passing it architecture and distro version
${SCRIPT_RSYNC_DESTINATION} -r ${SLACKWARE_RELEASE} -a ${ARCH} \
   | tee -a ${FILE_LOG} 2>&1

# Check for GPG Slackware key presence
GPG_KEY_REGISTERED=`gpg --list-keys | grep security@slackware.com | wc -l`
if [[ 1 -ne ${GPG_KEY_REGISTERED} ]]; then
   # Download and register the GPG key
   echo "GPG key download in progress" | tee -a ${FILE_LOG}
   wget ${GPG_KEY_SOURCE} -O ${PKG_TREE_ROOT}"/gpg-key"
   gpg --import ${PKG_TREE_ROOT}"/gpg-key"
   echo "GPG key successfully downloaded and registered" | tee -a ${FILE_LOG}
else
   echo "GPG key present" | tee -a ${FILE_LOG}
fi

# Check the list of the candidate for update packages
PKG_TREE_PKG=${PKG_TREE_ROOT}"/packages"
find ${PKG_TREE_PKG} -type f -name "*.t?z" | grep -iv kernel | sort \
   | while read PKG; do
   echo "Checking package ${PKG}" | tee -a ${FILE_LOG}
   upgradepkg --dry-run ${PKG} | tee -a ${FILE_LOG} 2>&1
   if [[ 0 -eq ${?} ]]; then
      echo "Checking package ${PKG} signature" | tee -a ${FILE_LOG}
      gpg --verify ${PKG}.asc | tee -a ${FILE_LOG} 2>&1
      if [[ 0 -eq ${?} ]]; then
         echo "Upgrading package ${PKG}" | tee -a ${FILE_LOG}
         upgradepkg ${PKG} | tee -a ${FILE_LOG} 2>&1
      else
         echo "Upgrade aborted due to issues with signature check" | tee -a ${FILE_LOG}
         exit -21
      fi
   fi
done

echo "${DATE_TIME} - Patch installation successfully concluded" | tee -a ${FILE_LOG}
exit 0