#!/bin/sh

# Script to distribute the downloaded source code for the Trinity Desktop
# Environment among the directories of the local build tree
#
# Copyright 2014 -2015 Simone Giustetti - Recco IT
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# -----------------------------------------------------------------------------
# Slackware Trinity source archive distribution script
# ========================================
# By:        Simone Giustetti <studiosg@giustetti.net>
# For:       TDE source archive distribute script
# Changelog:
# 1.0       2014/08/18 by Simone Giustetti <studiosg@giustetti.net>
#           * Initial build.
# 1.1       2014/08/21 by Simone Giustetti <studiosg@giustetti.net>
#           * Updated to the match the modifications in the download script.
#           * Package download directory is now read from 
#              source_download.conf file.
# 1.2       2014/10/07 by Simone Giustetti <studiosg@giustetti.net>
#           * Exit codes updated to comply with the new convention.
#           * Some bug fixes and minor enhancements.
# 2.0       2014/11/06 by Simone Giustetti <studiosg@giustetti.net>
#           * source_download.conf file removed in favor of the global
#              configuration file tde_build.conf.
# 2.1       2015/01/14 by Simone Giustetti <studiosg@giustetti.net>
#           * Added code to manage bzip2 archive file format.
#
# Run 'sh source_distribute.sh' to move source code archives into their proper
#   directory in the build tree.
#
# -----------------------------------------------------------------------------

function archive_check ()  {

   # Check for archives download directory existence or exit
   if [[ ! -d ${DIR_DOWNLOAD_ROOT} ]]; then
      echo "(EE) The archive download directory ${DIR_DOWNLOAD_ROOT} does not exist" | tee -a ${FILE_LOG}
      exit -41
   fi
   # Check for downloaded archive list files in the download directory or exit
   FILE_COUNT=`find ${DIR_DOWNLOAD_ROOT} -type f -size +42c -name "*.lst" \
      | wc -l`
   if [[ 0 -eq ${FILE_COUNT} ]]; then
      echo "(EE) No download list found in directory ${DIR_DOWNLOAD_ROOT}" | tee -a ${FILE_LOG}
      exit -42
   fi
   # Check for archives presence in the download directory or exit
   FILE_COUNT=`find ${DIR_DOWNLOAD_ROOT} -type f -size +1 \( -name "*.t?z" -o -name "*.xz" -o -name "*.bz2" \) \
      | wc -l`
   if [[ 0 -eq ${FILE_COUNT} ]]; then
      echo "(EE) No source archive found in directory ${DIR_DOWNLOAD_ROOT}" | tee -a ${FILE_LOG}
      exit -43
   fi
   echo "(II) Source archive check successsfully concluded" | tee -a ${FILE_LOG}
}

function arch_configuration_write ()  {
   # Write a local configuration file for GIT packages or packages with a
   #   release number different from the TDE one
   echo "(II) Configuration file writing started" | tee -a ${FILE_LOG}
   for COMPONENT in ${COMPONENT_ALL}; do
      if [[ "Y" = ${ARCHIVE_STATUS[${COMPONENT}]} ]]; then
         if [[ "GIT" = ${ARCHIVE_TYPE[${COMPONENT}]} ]]; then
            echo "SRCVER=\"master\"" > ${ARCHIVE_DESTINATION[${COMPONENT}]}"local.options"
            if [[ 0 -lt $? ]]; then
               echo "(EE) Impossible to write a configuration file for ${ARCHIVE_NAME[${COMPONENT}]}" | tee -a ${FILE_LOG}
            fi 
            # If RELEASE check if the TDE string is contained in the archive name
            # If not => Package like arts, koffice or similar => Find out the
            # Release number and write it down to the config file
            # This seems unlikely since release 3.5.13.2 uniformed the release
            # number to all packages
         fi
      fi
   done
   echo "(II) Configuration file writing concluded" | tee -a ${FILE_LOG}
}

# Move the source archives to their respective directory
function arch_distribute ()  {
   echo "(II) Archive distribution started" | tee -a ${FILE_LOG}
   for COMPONENT in ${COMPONENT_ALL}; do
      # Exclude zero lenght file
      if [[ -s ${ARCHIVE_SOURCE[${COMPONENT}]} ]]; then
         echo "(II) Moving ${ARCHIVE_SOURCE[${COMPONENT}]} to ${ARCHIVE_DESTINATION[${COMPONENT}]}" | tee -a ${FILE_LOG}
         mv ${ARCHIVE_SOURCE[${COMPONENT}]} ${ARCHIVE_DESTINATION[${COMPONENT}]}
         if [[ 0 -eq $? ]]; then
            ARCHIVE_STATUS[${COMPONENT}]="Y"
         else
            echo "(EE) ${ARCHIVE_SOURCE[${COMPONENT}]} move failed !" | tee -a ${FILE_LOG}
            if [[ ! -d ${ARCHIVE_DESTINATION[${COMPONENT}]} ]]; then
               echo "(EE) Directory ${ARCHIVE_DESTINATION[${COMPONENT}]} does not exist !" | tee -a ${FILE_LOG}
            fi
         fi
      else
         echo "(EE) Zero lenght file ${ARCHIVE_SOURCE[${COMPONENT}]} will be ignored" | tee -a ${FILE_LOG} 
      fi
   done
   echo "(II) Archive distribution concluded" | tee -a ${FILE_LOG}
}

# Read the package list from download direcotry
function arch_list_read ()  {
   # Create the destination directory
   DIR_TREE_SUBDIR=`basename ${ARCHIVE_LIST} | cut -d. -f1`
   DIR_DOWNLOAD_SOURCE=${DIR_DOWNLOAD_ROOT}"/"${DIR_TREE_SUBDIR}
   DIR_TREE_TARGET=${DIR_TREE_ROOT}"/"${DIR_TREE_SUBDIR}
   # Define arrays 1st component
   COUNTER_TOTAL=0
   # Load the package list from file ignoring comments
   while read PKG ARCH; do
      ARCHIVE_DESTINATION[${COUNTER_TOTAL}]=${DIR_TREE_TARGET}"/"${PKG}"/"
      ARCHIVE_NAME[${COUNTER_TOTAL}]=${ARCH}
      ARCHIVE_SOURCE[${COUNTER_TOTAL}]=${DIR_DOWNLOAD_SOURCE}"/"${ARCH}
      ARCHIVE_STATUS[${COUNTER_TOTAL}]="N"
      ARCHIVE_TYPE[${COUNTER_TOTAL}]=`arch_type_get ${ARCH}`
      # Component list update
      COMPONENT_ALL="${COMPONENT_ALL} ${COUNTER_TOTAL} "
      (( COUNTER_TOTAL=COUNTER_TOTAL+1 ))
   done <<< "$(cat ${ARCHIVE_LIST} | grep -v "#")"
}

function arch_type_get ()  {
   ARCH_NAME=$1
   # Check for parameter consistency
   if [[ "." = ${ARCH_NAME}"."  ]]; then
      echo "Archive name undefined. Please check the list" | tee -a ${FILE_LOG}
      exit -45
   fi
   GIT_FLAG=`echo ${ARCH_NAME} | grep master | wc -l`
   if [[ 1 -eq ${GIT_FLAG} ]]; then
      ARCH_TYPE="GIT"
   else
      ARCH_TYPE="RELEASE"
   fi
   echo ${ARCH_TYPE}
}

function configuration_read ()  {
   # Set the config option variables if they are not already set
   if [[ -r ${FILE_CONF} ]]; then
      echo "(II) Loading configuration from file ${FILE_CONF}" | tee -a ${FILE_LOG}
      . ${FILE_CONF}
   else
      echo "(EE) ${FILE_CONF} does not exist or is not readable" | tee -a ${FILE_LOG}
      echo "(II) Setting some standard values" | tee -a ${FILE_LOG}
   fi
   # Set some reasonable defaults in case the configuration file cannot be read
   DIR_DOWNLOAD_ROOT_ROOT=${DIR_DOWNLOAD_ROOT_ROOT:-"/tmp/tde"}
}

function tree_check ()  {
   # Check for build treee root directory existence or exit
   if [[ ! -d ${DIR_TREE_ROOT} ]]; then
      echo "(EE) The build tree directory ${DIR_DOWNLOAD_ROOT} does not exist" | tee -a ${FILE_LOG}
      exit -44
   fi
   echo "(II) Build tree check successsfully concluded" | tee -a ${FILE_LOG}
}


# Log variable declaration & initialization
DATE_TIME=`date +%Y%m%d`
FILE_LOG="/tmp/source_distribute-${DATE_TIME}.log"
# Log variable declaration & initialization ends here

# Script root directory (Declared because of chroot)
DIR_ROOT="/usr/src/trinity/tde_build_script/"

# Configuration file location
FILE_CONF="${DIR_ROOT}etc/tde_build.conf"

# Define arrays
declare -a ARCHIVE_DESTINATION
declare -a ARCHIVE_NAME
declare -a ARCHIVE_SOURCE
declare -a ARCHIVE_STATUS
declare -a ARCHIVE_TYPE
declare COMPONENT_ALL=""

####  Main  ####
# Init the log file
echo "${DATE_TIME} - Source code distribution in progress" | tee ${FILE_LOG}
# Move to the script direcotry
cd ${DIR_ROOT}
# Load configuration
configuration_read
# Check for file presence
archive_check
# Check for build tree presence
tree_check
# List source archive lists in the download root directory
ARCHIVE_LISTS=`find ${DIR_DOWNLOAD_ROOT} -type f -size +42c -name "*.lst"`
for ARCHIVE_LIST in ${ARCHIVE_LISTS}; do
   # Read archive name and destination
   arch_list_read
   # Distribute archives among the build tree directories generating a
   #    configuration file in the process
   arch_distribute
   # Create a configuration file for each archive build procedure
   arch_configuration_write
done

# Update the log file and exit
echo "${DATE_TIME} - Source code distribution successfully concluded" >> ${FILE_LOG}
exit 0