#!/bin/sh

# Script to download the source code for the Trinity Desktop Environment
#
# Copyright 2014 Simone Giustetti - Recco IT
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# -----------------------------------------------------------------------------
# Slackware Trinity source download script
# ========================================
# By:        Simone Giustetti <studiosg@giustetti.net>
# For:       TDE source download script
# Changelog:
# 1.0       2014/07/21 by Simone Giustetti <studiosg@giustetti.net>
#           * Initial build.
# 2.0       2014/08/19 by Simone Giustetti <studiosg@giustetti.net>
#           * Moved some parameters from the script main to the configuration
#              file.
#           * Added code to distribure archive pagages among download
#              directories (application, base, library, prerequisite).
#           * Added code to print out a list of successfully downloaded archives
#              in the download root directory.
#           * Removed configuration_dump and log_init functions.
# 2.1       2014/10/07 by Simone Giustetti <studiosg@giustetti.net>
#           * Exit codes updated to comply with the new convention.
#           * Added code to remove files from a partial download.
#           * Some bug fixes and minor enhancements.
# 3.0       2014/11/06 by Simone Giustetti <studiosg@giustetti.net>
#           * source_download.conf file removed in favor of the global
#              configuration file tde_build.conf.
#           * Variable ARCHIVE_ORIGIN replaced ARCHIVE_SOURCE.
#
# Run 'sh source_download.sh' to download the source code archives for TDE.
#
# -----------------------------------------------------------------------------

function configuration_check ()  {
   # Check for download URL to exist or exit
   if [[ "RELEASE" = ${ARCHIVE_ORIGIN} ]]; then
      # Cuncatenate a / otherwise curl will mistake the directory for a file
      PKG_DOWNLOAD_URL=${RELEASE_BASE_URL}"/"${TDE_RELEASE}"/"
      curl --head -s ${PKG_DOWNLOAD_URL}
      if [[ 0 -lt ${?} ]]; then
         echo "(EE) Error while downloading files for a RELEASE build" | tee -a ${FILE_LOG}
         echo "(EE) Unable to retrieve URL ${PKG_DOWNLOAD_URL}" | tee -a ${FILE_LOG}
         echo "(EE) Aborting download. Please check the configuration file" | tee -a ${FILE_LOG}
         exit -31
      fi
   fi
}

function configuration_read ()  {
   # Set the config option variables if they are not already set
   if [[ -r ${FILE_CONF} ]]; then
      echo "(II) Loading configuration from file ${FILE_CONF}" | tee -a ${FILE_LOG}
      . ${FILE_CONF}
   else
      echo "(EE) ${FILE_CONF} does not exist or is not readable" | tee -a ${FILE_LOG}
      echo "(II) Setting some standard values" | tee -a ${FILE_LOG}
   fi
   # Set some reasonable defaults in case configuration file cannot be read
   ARCHIVE_ORIGIN=${ARCHIVE_ORIGIN:-"RELEASE"}
   BUILD_TYPE=${BUILD_TYPE:-"FULL"}
   TDE_RELEASE=${TDE_RELEASE:-"R14.0.0"}
   PKG_APP_LIST=${PKG_APP_LIST:-"${DIR_ROOT}etc/tde_pkg_application.lst"}
   PKG_BASE_LIST=${PKG_BASE_LIST:-"${DIR_ROOT}etc/tde_pkg_base.lst"}
   PKG_LIB_LIST=${PKG_LIB_LIST:-"${DIR_ROOT}etc/tde_pkg_library.lst"}
   PKG_PRE_LIST=${PKG_PRE_LIST:-"${DIR_ROOT}etc/tde_pkg_prerequisite.lst"}
   RELEASE_BASE_URL=${RELEASE_BASE_URL:-"ftp://ftp.fau.de/trinity/releases"}
   GIT_BASE_URL=${GIT_BASE_URL:-"https://git.trinitydesktop.org/cgit"}
   GIT_BRANCH=${GIT_BRANCH:-"master"}
   DIR_DOWNLOAD_ROOT=${DIR_DOWNLOAD_ROOT:-"/tmp/tde"}
}

function pkg_download ()  {
   # Select download directory
   case ${BUILD_TARGET} in
      APP)
         DIR_DOWNLOAD_NAME="application"
      ;;
      BASE)
         DIR_DOWNLOAD_NAME="base"
      ;;
      LIB)
         DIR_DOWNLOAD_NAME="library"
      ;;
      PRE)
         DIR_DOWNLOAD_NAME="prerequisite"
      ;;
      *)
         echo "(EE) ${BUILD_TARGET} is not a valid build target. Please check the configuration file" | tee -a ${FILE_LOG}
         exit -36
      ;;
   esac
   DIR_DOWNLOAD=${DIR_DOWNLOAD_ROOT}"/"${DIR_DOWNLOAD_NAME}
   if [[ ! -d ${DIR_DOWNLOAD} ]]; then
      echo "Download directory not fond. Creating it" | tee -a ${FILE_LOG}
      mkdir -p ${DIR_DOWNLOAD}
   fi
   # Assign sane permissions to the download directory
   chmod 0744 ${DIR_DOWNLOAD}
   # Remove the remnants of previous downloads
   rm -f ${DIR_DOWNLOAD}/*
   # Download source archives
   for COMPONENT in ${COMPONENT_ALL}; do
      echo "(II) Downloading ${LIST_PACKAGE_NAME[${COMPONENT}]} from ${LIST_PACKAGE_SOURCE[${COMPONENT}]}" | tee -a ${FILE_LOG}
      wget -nv -T 20 -P "${DIR_DOWNLOAD}/" "${LIST_PACKAGE_SOURCE[${COMPONENT}]}"
      if [[ 0 = $? ]]; then
         LIST_PACKAGE_STATUS[${COMPONENT}]="Y"
         echo "(II) ${LIST_PACKAGE_NAME[${COMPONENT}]} successfully downloaded" | tee -a ${FILE_LOG}
      else
         echo "(WW) Problems detected while downloading ${LIST_PACKAGE_NAME[${COMPONENT}]}" | tee -a ${FILE_LOG}
      fi
   done
   # Save a list of downloaded packages
   pkg_list_outcome ${DIR_DOWNLOAD_NAME}
}

function pkg_list_check ()  {
   # Check for packages in the repository. If not present switch to GIT
   if [[ "RELEASE" = ${ARCHIVE_ORIGIN} ]]; then
      for COMPONENT in ${COMPONENT_ALL}; do
         curl --head -s ${LIST_PACKAGE_SOURCE[${COMPONENT}]}
         if [[ 0 -lt ${?} ]]; then
            echo "(EE) Unable to find ${LIST_PACKAGE_NAME[${COMPONENT}]} in the repository" | tee -a ${FILE_LOG}
            echo "(II) Switching to a GIT download " | tee -a ${FILE_LOG}
            LIST_PACKAGE_SOURCE[${COMPONENT}]=`pkg_url_download ${LIST_PACKAGE_NAME[${COMPONENT}]} "GIT"`
         fi
      done
   fi
}

# Write a list of downloaded packages
function pkg_list_outcome ()  {
   PKG_OUTPUT_LIST=${1}

   # Check for argument consistency
   if [[ ${PKG_OUTPUT_LIST}"." = "." ]]; then
      echo "(EE) No output list provided" | tee -a ${FILE_LOG}
      exit -37
   fi
   # Check for package list file to exist and be writeable
   FILE_OUTPUT=${DIR_DOWNLOAD_ROOT}"/"${PKG_OUTPUT_LIST}".lst"
   if [[ ! -w ${FILE_OUTPUT} ]]; then
      touch ${FILE_OUTPUT}
      chmod 0644 ${FILE_OUTPUT}
   fi
   # Init the file list
   echo "# Downloaded packages for the ${PKG_OUTPUT_LIST} subcategory" > ${FILE_OUTPUT}
   for COMPONENT in ${COMPONENT_ALL}; do
      if [[ "Y" = ${LIST_PACKAGE_STATUS[${COMPONENT}]} ]]; then
         ARCHIVE_NAME=`echo ${LIST_PACKAGE_SOURCE[${COMPONENT}]} | awk -F/ '{ print $NF }'`
         echo ${LIST_PACKAGE_NAME[${COMPONENT}]}" "${ARCHIVE_NAME} >> ${FILE_OUTPUT}
      fi
   done
}

# Read the package list from the configuration files and build a download
# URL for each package
function pkg_list_read ()  {
   if [[ "." = ${PKG_LIST}"."  ]]; then
      echo "Package list undefined. Please check variable declaration" | tee -a ${FILE_LOG}
      exit -33
   fi
   # Check for package list file
   if [[ ! -r ${PKG_LIST} ]]; then
      echo "(EE) ${PKG_LIST} does not exist or is not readable" | tee -a ${FILE_LOG}
      exit -34
   fi
   # Define arrays 1st component
   COUNTER_TOTAL=0
   # Load the package list from file ignoring comments
   while read PKG; do
      LIST_PACKAGE_NAME[${COUNTER_TOTAL}]=${PKG}
      LIST_PACKAGE_SOURCE[${COUNTER_TOTAL}]=`pkg_url_download ${PKG} ${ARCHIVE_ORIGIN}`
      LIST_PACKAGE_STATUS[${COUNTER_TOTAL}]="N"
      # Component list update
      COMPONENT_ALL="${COMPONENT_ALL} ${COUNTER_TOTAL} "
      (( COUNTER_TOTAL=COUNTER_TOTAL+1 ))
   done <<< "$(cat ${PKG_LIST} | grep -v "#")"
}

# Create a download URL for a package based on source archive type
function pkg_url_download ()  {
   URL_DOWNLOAD=""
   PKG=${1}
   SOURCE=${2}

   # Check for argument consistency
   if [[ ${PKG}"." = "." ]]; then
      echo "(EE) No package provided. Download URL undefined" | tee -a ${FILE_LOG}
   else
      # Choose an URL based on the configuration
      case ${SOURCE} in
         GIT)
            PKG_NAME=${PKG}"-"${GIT_BRANCH}".tar.gz"
            URL_DOWNLOAD=${GIT_BASE_URL}"/"${PKG}"/snapshot/"${PKG_NAME}
         ;;
         RELEASE)
            # Write the package name translating TDE in KDE. Something the GIT
            # download does not need
            PKG_NAME=`echo ${PKG} | sed s/tde/kde/g`
            PKG_NAME=${PKG_NAME}"-trinity-"${TDE_RELEASE}".tar.xz"
            # Determine the subdirectory when needed
            case ${BUILD_TARGET} in
               APP)
                  PKG_DOWNLOAD_URL_SUFFIX="applications/"
               ;;
               LIB)
                  PKG_DOWNLOAD_URL_SUFFIX="libraries/"
               ;;
               PRE)
                  PKG_DOWNLOAD_URL_SUFFIX="dependencies/"
               ;;
               *)
                  # Works for base build packages too
                  PKG_DOWNLOAD_URL_SUFFIX=""
               ;;
            esac
            # Prepend the base address
            URL_DOWNLOAD=${RELEASE_BASE_URL}"/"${TDE_RELEASE}"/"${PKG_DOWNLOAD_URL_SUFFIX}${PKG_NAME}
         ;;
         *)
            echo "(EE) Unknown archive source ${SOURCE}. Please check the configuration" | tee -a ${FILE_LOG}
            exit -35
         ;;
      esac
   fi
   echo ${URL_DOWNLOAD}
}


# Log variable declaration & initialization
DATE_TIME=`date +%Y%m%d`
FILE_LOG="/tmp/source_download-${DATE_TIME}.log"
# Log variable declaration & initialization ends here

# Script root directory (Declared because of chroot)
DIR_ROOT="/usr/src/trinity/tde_build_script/"

# Configuration file location
FILE_CONF="${DIR_ROOT}etc/tde_build.conf"

# Define arrays
declare -a LIST_PACKAGE_NAME
declare -a LIST_PACKAGE_SOURCE
declare -a LIST_PACKAGE_STATUS
declare COMPONENT_ALL=""

#### Main ####
# Init the log file
echo "${DATE_TIME} - Source code download in progress" | tee ${FILE_LOG}
# Move to the script direcotry
cd ${DIR_ROOT}
# Read the configuration file
configuration_read
# Write the loaded parameters to file
echo "(II) Downloading source archives for TDE ${TDE_RELEASE} in order to perform a
   ${ARCHIVE_ORIGIN} build for the ${BUILD_TYPE} packages" | tee -a ${FILE_LOG}
# Check for release download URL
configuration_check
# Impone the correct order for actions to be performed
case ${BUILD_TYPE} in
   APP)
      BUILD_TARGET=${BUILD_TYPE}
      PKG_LIST=${PKG_APP_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
   ;;
   BASE)
      BUILD_TARGET=${BUILD_TYPE}
      PKG_LIST=${PKG_BASE_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
   ;;
   FULL)
      BUILD_TARGET="PRE"
      PKG_LIST=${PKG_PRE_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
      BUILD_TARGET="BASE"
      PKG_LIST=${PKG_BASE_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
      BUILD_TARGET="LIB"
      PKG_LIST=${PKG_LIB_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
      BUILD_TARGET="APP"
      PKG_LIST=${PKG_APP_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
   ;;
   LIB)
      BUILD_TARGET=${BUILD_TYPE}
      PKG_LIST=${PKG_LIB_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
   ;;
   PRE)
      BUILD_TARGET=${BUILD_TYPE}
      PKG_LIST=${PKG_PRE_LIST}
      pkg_list_read
      pkg_list_check
      pkg_download
   ;;
   *)
      echo "(EE) ${BUILD_TYPE} is not a recognized operation" | tee -a ${FILE_LOG}
      echo "(EE) Aborting download. Please check the configuration file" | tee -a ${FILE_LOG}
      exit -32
   ;;
esac

echo "${DATE_TIME} - Source code download successfully concluded" | tee -a ${FILE_LOG}
exit 0